@@ -16,13 +16,13 @@ given the generator C< my $gen = gen {2**$_} 100; > which computes the first
 hundred powers of two, here are a few was to iterate over it (that all maintain
 lazy evaluation):
 
-    for (@$gen) {...}      # no need to reset generator between calls
+    for (@$gen) {...}       # no need to reset generator between calls
     for my $p (@$gen) {...}
     ... for @$gen;
 
-    while (<$gen>) {...}   # iterated generators must be reset with `$gen->reset`
-    ... while <$gen>       # before each loop, also be sure to `local $_` before
-                           # while loops that modify `$_`
+    while (<$gen>) {...}    # generator must be reset with `$gen->reset`
+    ... while <$gen>        # between calls, also be sure to `local $_` before
+                            # while loops that modify `$_`
     while (my $p = <$gen>) {...}
     while (defined(my $p = $gen->())) {...}
     while ($gen->more) {do something with $gen->next}
@@ -30,10 +30,6 @@ lazy evaluation):
 since all of these iteration examples remain lazy (only generating values on
 demand), you can C< last > at any time to break out of the loop.
 
-you can also use the C< do > method:
-
-    $gen->do(sub {...}); # which calls sub on every element of $gen
-
 =head2 list creation
 
 you can dereference finite length generators to pass all of their elements to
@@ -67,10 +63,38 @@ on:
 
     for (@{ +gen {$_**2} 1, 10 }) {...}  # a '+' or ';' before it does the trick
 
+=head2 glob syntax
+
+if you export the C< glob > function from L<List::Gen>, that function and the
+C<< <*.glob> >> operator will have one special case overridden.  if given an
+argument that matches the following pattern:
+
+   /^ ( .+ : )? number .. number ( (by | += | -= | [-+,]) number )?
+                                 ( (if | when) .+ )? $/
+
+then the arguments will be passed to C< range >, C< gen >, and C< filter > as
+appropriate. any argument that doesn't match that pattern will be passed to
+perl's builtin C< glob > function.  here are a few examples:
+
+    <1 .. 10>                    ~~  range 1, 10
+    <1 .. 10 by 2>               ~~  range 1, 10, 2
+    <10 .. 1 -= 2>               ~~  range 10, 1, -2
+    <x * x: 1 .. 10>             ~~  gen {$_ * $_} 1, 10
+    <sin: 0 .. 3.14 += 0.01>     ~~  gen {sin} 0, 3.14, 0.01
+    <1 .. 10 if x % 2>           ~~  filter {$_ % 2} 1, 10
+    <sin: 0 .. 100 by 3 if /5/>  ~~  filter {/5/} gen {sin} 0, 100, 3
+
+    for (@{< 0 .. 1_000_000_000 by 2 >}) { # starts instantly
+        print "$_\n";
+        last if $_ >= 100;        # exits the loop with only 51 values generated
+    }
+
+    my @files = <*.txt>;  # works as normal
+
 =head2 normal generators
 
 the C< range > and C< makegen > functions are the most primitive generators,
-C< range > producing a lazy list, and C< makegen > wrapping a perl array.
+C< range > producing a lazy list, and C< makegen > storing an array.
 
 you build upon these with the other generator functions/methods.  many
 generator functions will pass their arguments along to C< range > or
@@ -81,7 +105,7 @@ C< makegen > as needed, so you rarely need to use them directly.
     my @names = qw/bob alice eve/;
     gen {"hello $_!"} \@names   ~~  gen {"hello $_!"} makegen @names
 
-those were two examples of C< gen >, the generator equivalent of C< map > that
+those were two examples of C< gen > the generator equivalent of C< map > that
 attaches a code block to a generator.
 
 =head2 iterative generators
@@ -106,11 +130,6 @@ in two flavors, single element per iteration, and multi element per iteration.
         }
     }
 
-you can also use the C<< ->from >> method to write an iterator that builds
-from an initial value:
-
-    say iterate{$_*2}->from(1)->str(10); # 1 2 4 8 16 32 64 128 256 512
-
 the iterative generators have some syntactic sugar you can use, in the form of
 C< gather {...} > and C< take(...) >:
 
@@ -246,156 +265,14 @@ of the fibonacci sequence would generate an exponentially increasing number of
 calls to itself as the list grows longer.  C< cache > prevents any index from
 being calculated more than once.
 
-=head3 more ways to write the fibonacci sequence
-
-    my $fib = <0, 1, *+*...>; >>
-
-    my $fib = <0, 1, {$^a + $^b}...>; >>
-
-    my $fib = ([0, 1] + iterate {sum self($_, $_ + 1)})->rec; >>
-
-    my $fib = ([0, 1] + iterate {sum fib($_, $_ + 1)})->rec('fib'); >>
-
-    my $fib = (iterate {$_ < 2 ? $_ : sum self($_ - 1, $_ - 2)})->rec; >>
-
-    my $fib; $fib = cache gen {$_ < 2 ? $_ : sum $fib->($_ - 1, $_ - 2)}; >>
-
-=head3 a few ways to write the factorial sequence
-
-    my $fac = <[*..] 1, 1..>;
-
-    my $fac = <1, 1..>->scan('*');
-
-    my $fac = 1 + repeat(1)->scan('+')->scan('*');
-
-    my $fac = <1, **_...>;
-
-=head2 stream generators
-
-here is an example of a sieve of eratosthenes implemented with generators:
-
-    my $primes = do {
-        my $src = <2..>;
-        iterate {
-            my ($x, $xs) = $src->x_xs;
-            $src = $xs->grep_stream(sub {$_ % $x});
-            $x
-        }
-    };
-
-in this example, the list is filtered with C< grep_stream/filter_stream > since
-the algorithm only reads the source once, and reads it in order.  a regular
-C< filter/grep > call could be used, but it would unnecessarily use up a lot
-of memory since each call would have to build up a new random-access cache.
-
-the inefficiency addressed above could also be fixed by modifying the filtering
-function itself:
-
-    my $primes = do {
-        my @p;
-        <2..>->grep(sub {
-            my $i = $_;
-            $i % $_ or return for @p;
-            push @p, $i;
-        })
-    };
-
-of course if you want prime numbers, just use the C< primes > function:
-
-    my $primes = List::Gen::primes;
-
-which is implemented as a precomputed sieve of eratosthenes in a string buffer.
-initially it is ready to test the primality of numbers below 1000.  if a higher
-number is checked, the sieve will grow to 10 times larger than that value.
-beyond 1e7 primes are checked with simple trial division.
-
-=head2 printing generators
-
-there are a variety of methods available for printing out the contents of a
-generator:
-
-    my $gen = <1..5>;
-
-    say $gen->str;  # 1 2 3 4 5
-    $gen->say;      # 1 2 3 4 5
-    $gen->print;    # same as: print $gen;
-
-    say $gen->perl; # [1, 2, 3, 4, 5]
-    $gen->dump;     # [1, 2, 3, 4, 5]
-
-if your generator is longer than you would like to print, such as an infinite
-generator, passing a number to any of the methods above will limit the number
-of elements printed.
-
-    <1..>->say(5); # 1 2 3 4 5
-
-which is the same as
-
-    <1..>->take(5)->say;  # 1 2 3 4 5
-
-if passed an additional argument, that string will be included in the output
-whenever the printing method needs to truncate a generator.
-
-    say <1..>->perl(5, '...');  # [1, 2, 3, 4, 5, ...]
-
-these methods are recursive and will expand elements that are generators or
-array references.
-
-    list(<1..>, <a..>, <A..>)->dump(3, '...');
-
-    # [[1, 2, 3, ...], ['a', 'b', 'c', ...], ['A', 'B', 'C', ...]]
-
-    <1..>->tuples(<a..>)->dump(3); # [[1, 'a'], [2, 'b'], [3, 'c']]
-
-a target file handle can be passed as the first argument:
-
-    <1..>->dump(*STDERR, 5);
-
-the C<say>, C<print>, and C<dump> methods all return the generator they were
-called on for easy chaining.
-
-    <1..>->say(5)->map('**2')->say(5);
-    # 1 2 3 4 5
-    # 1 4 9 16 25
-
-=head2 debugging generators
-
-in addition to the methods to print generators, there are several methods
-dedicated to debugging:
-
-    <0..>->debug;
-    # debug:   List::Gen::erator::_20=ARRAY(0x2d07bfc)
-    # type:    List::Gen::Range
-    # source:  none
-    # mutable: no
-    # stream:  no
-    # range:   [0 .. inf]
-    # index:   0
-    # perl:    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, ...]
-    #  at file.pl line 12
-
-pass C< debug > a number to control how many elements are printed.
-
-    my $gen = <0..>->watch('range')
-                   ->grep('even')->watch('grep')
-                   ->map('**2')->watch('map')
-                   ->map('"[$_]"');
-
-    local $\ = ', '; # watch ends lines with $\ if defined or with $/
-
-    say $gen->(0); # range: 0, range: 1, range: 2, grep: 0, map: 0, [0]
-    say $gen->(1); # range: 3, range: 4, grep: 2, map: 4, [4]
-    say $gen->(2); # range: 5, range: 6, grep: 4, map: 16, [16]
-
-C<watch> can also be passed a file handle to print to.
 
 =head1 AUTHOR
 
-Eric Strom, C<< <asg at cpan.org> >>
+Eric Strom, C<< <ejstrom at gmail.com> >>
 
 =head1 COPYRIGHT & LICENSE
 
-copyright 2009-2011 Eric Strom.
+copyright 2009-2010 Eric Strom.
 
 this program is free software; you can redistribute it and/or modify it under
 the terms of either: the GNU General Public License as published by the Free
@@ -405,5 +282,4 @@ see http://dev.perl.org/licenses/ for more information.
 
 =cut
 
-
 __PACKAGE__ if 'first require';
